using System;
using System.Net;
using System.Collections.Generic;
using System.Text;

namespace KSL
{
    // updates the DynDNS config for the account and hostnames to the ip address
    // assumes the ip address has already been checked and needs updating
    public class DynDNSUpdater
    {
        public static string CheckIP()
        {

            // check the current public IP address
            string ipString = string.Empty;
            WebClient wc = new WebClient();
            try
            {
                string result = wc.DownloadString("http://checkip.dyndns.com");
                return ParseCheckResult(result);
            }
            catch (WebException)
            {
                ipString = string.Empty;
            }
            return ipString;

        }

        public static string UpdateIP(string username, string password, string ipaddr, string hosts)
        {
            // check the current public IP address against what we want to update to
            string updateurl = "members.dyndns.org/nic/update";
            string result = string.Empty;
            WebClient wc = new WebClient();

            string url = string.Format(@"http://{0}?hostname={1}&myip={2}&wildcard=NOCHG&mx=NOCHG&backmx=NOCHG", updateurl, hosts, ipaddr);
            try
            {
                wc.Credentials = new NetworkCredential(username, password);
                wc.Headers.Add("User-Agent", "KSL - WHS Updater - 1.0");
                result = ParseUpdateResult(wc.DownloadString(url));
            }
            catch (WebException we)
            {
                System.Diagnostics.Trace.WriteLine(we.ToString());
                result = ParseUpdateResult("badauth");
            }

            System.Diagnostics.Trace.WriteLine(result);
            return result;
        }

        public static string UpdateIP(string username, string password, string hosts)
        {
            // check the current public IP address against what we want to update to
            string result = string.Empty;
            string ipaddr = CheckIP();
            if (!(string.Empty == ipaddr))
            {
                result = UpdateIP(username, password, ipaddr, hosts);
            }
            else
            {
                result = "noip";
            }

            System.Diagnostics.Trace.WriteLine(result);
            return result;

        }

        private static string ParseCheckResult(string result)
        {
            string temp = result.Substring(result.IndexOf(":") + 2);
            temp = temp.Remove(temp.IndexOf("<"));
            return temp;
        }

        private static string ParseUpdateResult(string result)
        {
            string[] resultCodes = { "good", "badauth", "nochg", "!donator", "nohost", "not fqdn", "numhost", "abuse", "badagent", "dnserr", "911" };
            string parsedResult = string.Empty;
            foreach (string resultCode in resultCodes)
            {
                if (result.ToLower().Contains(resultCode))
                {
                    parsedResult = resultCode;
                    break;
                }
            }

            if (parsedResult == string.Empty)
                parsedResult = result;

            return parsedResult;
        }

        public static string GetDynDNSErrorDescription(string resultCode)
        {
            string errorDescription = string.Empty;
            switch (resultCode)
            {
                case "good":
                    errorDescription = "Successfully Updated";
                    break;

                case "badauth":
                    errorDescription = "Invalid authorization credentials";
                    break;

                case "nochg":
                    errorDescription = "No change required";
                    break;

                case "!donator":
                    errorDescription = "This feature is only available to credited users";
                    break;

                case "nohost":
                    errorDescription = "The hostname does not exist in this user account";
                    break;

                case "not fqdn":
                    errorDescription = "The hostname is not a Fully Qualified Domain Name";
                    break;

                case "numhost":
                    errorDescription = "Too many hosts (more than 20) in one update";
                    break;

                case "abuse":
                    errorDescription = "The hostname is blocked for update abuse";
                    break;

                case "dnserr":
                    errorDescription = "DNS error encountered";
                    break;

                case "911":
                    errorDescription = "There is a problem (or scheduled maintenance at DynDNS)";
                    break;

                case "badagent":
                    errorDescription = "Invalid user agent specified";
                    break;

                case "noip":
                    errorDescription = "Could not find IP address";
                    break;

                default:
                    errorDescription = "Unknown result code, could not be parsed";
                    break;
            }

            return errorDescription;
        }
    }
}
